'''
Created on Sep 1, 2017

@author: sumanth-3058
'''
try:
    from .RestClient import ZCRMRestClient
    from .Operations import ZCRMAttachment,ZCRMCustomView,ZCRMCustomViewCategory,ZCRMCustomViewCriteria,ZCRMEventParticipant,ZCRMField,ZCRMInventoryLineItem,ZCRMJunctionRecord,ZCRMLayout,ZCRMLeadConvertMapping,ZCRMLeadConvertMappingField,ZCRMLookupField,ZCRMModule,ZCRMModuleRelatedList,ZCRMModuleRelation,ZCRMNote,ZCRMPermission,ZCRMPickListValue,ZCRMPriceBookPricing,ZCRMProfile,ZCRMProfileCategory,ZCRMProfileSection,ZCRMRecord,ZCRMRelatedListProperties,ZCRMRole,ZCRMSection,ZCRMTax,ZCRMTrashRecord,ZCRMUser,ZCRMUserCustomizeInfo,ZCRMUserTheme,ZCRMVariable,ZCRMVariableGroup,ZCRMBlueprint,ZCRMBlueprintRelatedModule,ZCRMBlueprintTransition,ZCRMTransitionAttachment,ZCRMTransitionCheckList,ZCRMTransitionRelatedData,ZCRMTransition,ZCRMTransitionCrypt,ZCRMOrgTax,ZCRMTag
    from .Org import ZCRMOrganization
    from .CLException import ZCRMException
    from .Utility import APIConstants
    from .OAuthClient import ZohoOAuth,ZohoOAuthClient,ZohoOAuthTokens,AbstractZohoOAuthPersistence
except ImportError:
    from RestClient import ZCRMRestClient
    from Operations import ZCRMAttachment,ZCRMCustomView,ZCRMCustomViewCategory,ZCRMCustomViewCriteria,ZCRMEventParticipant,ZCRMField,ZCRMInventoryLineItem,ZCRMJunctionRecord,ZCRMLayout,ZCRMLeadConvertMapping,ZCRMLeadConvertMappingField,ZCRMLookupField,ZCRMModule,ZCRMModuleRelatedList,ZCRMModuleRelation,ZCRMNote,ZCRMPermission,ZCRMPickListValue,ZCRMPriceBookPricing,ZCRMProfile,ZCRMProfileCategory,ZCRMProfileSection,ZCRMRecord,ZCRMRelatedListProperties,ZCRMRole,ZCRMSection,ZCRMTax,ZCRMTrashRecord,ZCRMUser,ZCRMUserCustomizeInfo,ZCRMUserTheme,ZCRMVariable,ZCRMVariableGroup,ZCRMBlueprint,ZCRMBlueprintRelatedModule,ZCRMBlueprintTransition,ZCRMTransitionAttachment,ZCRMTransitionCheckList,ZCRMTransitionRelatedData,ZCRMTransition,ZCRMTransitionCrypt,ZCRMOrgTax,ZCRMTag
    from Org import ZCRMOrganization
    from CLException import ZCRMException
    from Utility import APIConstants
    from OAuthClient import ZohoOAuth,ZohoOAuthClient,ZohoOAuthTokens,AbstractZohoOAuthPersistence